entrez_to_symbol <- function(gse_matrix_df, col_to_convert = "pl_annot", symbol_to_symbol = FALSE){
  entrez_to_symbol_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/symbol-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )
  
  if (symbol_to_symbol) {
    m <- match(gse_matrix_df[, col_to_convert], entrez_to_symbol_df[, 3])
    gse_matrix_df$symbol <- gse_matrix_df$pl_annot
  } else {
    m <- match(gse_matrix_df[, col_to_convert], entrez_to_symbol_df[, 2])
    gse_matrix_df$symbol <- entrez_to_symbol_df[m, 3]
  }

  gse_matrix_df$entrez <- entrez_to_symbol_df[m, 2]
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$symbol), ]
  return(gse_matrix_df)
}


ensembl_to_symbol <- function(gse_matrix_df) {
  ensembl_to_entrez_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/ensembl-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )

  m <- match(gse_matrix_df$pl_annot, ensembl_to_entrez_df[, 3])

  gse_matrix_df <- data.frame(gse_matrix_df, stringsAsFactors = F)
  gse_matrix_df$entrez_id <- ensembl_to_entrez_df[m, 2]
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ]

  gse_matrix_df <- entrez_to_symbol(gse_matrix_df, col_to_convert = "entrez_id")
  gse_matrix_df <- gse_matrix_df[, !grepl("entrez_id", colnames(gse_matrix_df))]
  return(gse_matrix_df)
}


# 5th element of 'gene_assignment' column is the entrez id
gene_assign_to_symbol <- function(gse_matrix_df) {
  gse_matrix_df$entrez_id <- sapply(gse_matrix_df$pl_annot, function(x) {
    id_vec <- unlist(sapply(strsplit(x, split = " /// ")[[1]], function(x) strsplit(x, split = " // ")[[1]][5]))
    return(paste(unique(id_vec[id_vec != "---"]), collapse = ","))
  })
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ]
  gse_matrix_df <- gse_matrix_df[gse_matrix_df$entrez_id != "", ]
  
  gse_matrix_df <- entrez_to_symbol(gse_matrix_df, col_to_convert = "entrez_id")
  gse_matrix_df <- gse_matrix_df[, !grepl("entrez_id", colnames(gse_matrix_df))]
  return(gse_matrix_df)
}

# 6th element of 'Associated Gene' column is the entrez id
gene_assoc_to_symbol <- function(gse_matrix_df) {
    gse_matrix_df$entrez_id <- sapply(gse_matrix_df$pl_annot, function(x) {
    id_vec <- unlist(sapply(strsplit(x, split = " /// ")[[1]], function(x) strsplit(x, split = " // ")[[1]][6]))
    return(paste(unique(id_vec[id_vec != "---"]), collapse = ","))
  })
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ]
  gse_matrix_df <- gse_matrix_df[gse_matrix_df$entrez_id != "", ]

  gse_matrix_df <- entrez_to_symbol(gse_matrix_df, col_to_convert = "entrez_id")
  gse_matrix_df <- gse_matrix_df[, !grepl("entrez_id", colnames(gse_matrix_df))]
  return(gse_matrix_df)
}


gb_acc_to_symbol <- function(gse_matrix_df) {
  refseq_to_entrez_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/refseq-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )
  embl_to_entrez_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/embl-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )
  gse_matrix_df$pl_annot <- sapply(gse_matrix_df$pl_annot, function(x) strsplit(x, split="\\.")[[1]][1])
  gse_matrix_df$pl_annot <- sapply(gse_matrix_df$pl_annot, function(x) strsplit(x, split="\\,")[[1]][1])

  m <- match(gse_matrix_df$pl_annot, refseq_to_entrez_df[, 3])
  gse_matrix_df$entrez_id <- refseq_to_entrez_df[m, 2]
  if (nrow(gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ])==0){
      match_embl <- match(gse_matrix_df$pl_annot, embl_to_entrez_df[, 3])
      gse_matrix_df$entrez_id <- embl_to_entrez_df[match_embl, 2]
  }
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ]                                 
  gse_matrix_df <- gse_matrix_df[gse_matrix_df$entrez_id != "", ]
  if (nrow(gse_matrix_df)==0)
      stop("ERROR: annotation column does not follow RefSeq database accession number format")
  gse_matrix_df <- entrez_to_symbol(gse_matrix_df, col_to_convert = "entrez_id")
  gse_matrix_df <- gse_matrix_df[, !grepl("entrez_id", colnames(gse_matrix_df))]
  if (nrow(gse_matrix_df)==0){
      message("ERROR: mapped entrez ids were not found in annotation files")
  } else {
      return(gse_matrix_df)
  }
}                                        

gb_list_to_symbol <- function(gse_matrix_df) {
  refseq_to_entrez_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/refseq-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )
  embl_to_entrez_df <- read.table(paste(path, "/gpl_auto_annotation/annotation_files/embl-to-entrez.tsv", sep =""),
    sep = "\t", stringsAsFactors = F
  )
  gse_matrix_df$pl_annot <- sapply(gse_matrix_df$pl_annot, function(x) strsplit(x, split="\\.")[[1]][1])
  gse_matrix_df$pl_annot <- sapply(gse_matrix_df$pl_annot, function(x) strsplit(x, split="\\,")[[1]][1])

  m <- match(gse_matrix_df$pl_annot, refseq_to_entrez_df[, 3])
  gse_matrix_df$entrez_id <- refseq_to_entrez_df[m, 2]
  if (nrow(gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ])==0){
      match_embl <- match(gse_matrix_df$pl_annot, embl_to_entrez_df[, 3])
      gse_matrix_df$entrez_id <- embl_to_entrez_df[match_embl, 2]
  }
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$entrez_id), ]
  gse_matrix_df <- gse_matrix_df[gse_matrix_df$entrez_id != "", ]
  if (nrow(gse_matrix_df)==0)
    stop("ERROR: annotation column does not follow RefSeq database accession number format")
  gse_matrix_df <- entrez_to_symbol(gse_matrix_df, col_to_convert = "entrez_id")
  gse_matrix_df <- gse_matrix_df[, !grepl("entrez_id", colnames(gse_matrix_df))]
  if (nrow(gse_matrix_df)==0){
      message("ERROR: mapped entrez ids were not found in annotation files")
  } else {
      return(gse_matrix_df)
  }
}  


probe_to_gene <- function(gse_matrix_df, gpl) {
  
  library("AnnoProbe")
  probe2gene=idmap(gpl,type = 'pipe')
  
  m <- match(gse_matrix_df$pl_annot, probe2gene[,'probe_id'])
  gse_matrix_df$symbol <- probe2gene[m, 'symbol']
  gse_matrix_df <- gse_matrix_df[!is.na(gse_matrix_df$symbol), ]
  return(gse_matrix_df)
  
  file.remove(paste(gpl,'_pipe.rda'))
}